const functional = module.exports = {};

/*

	OUTCOME

	logs the input to the console, but is also chainable

 */

functional.log = function(x) {
	console.log(x);
	return x;
};


/*

	OUTCOME

	calls .map on its first argument and returns the result
	
	USAGE

	const add3 = (n) => n + 3;

	map(add3)(Maybe(5)) // calls add3 with 5 as argument
	map(add3)(Maybe(null)) // does nothing, add3 is never called
	
	map(add3)([1, 2, 3, 4, 5]) // returns [4, 5, 6, 7, 8]

*/

functional.map = (fn) => (container) => container.map(fn);


/*

	OUTCOME

	creates a function that is the result of calling the arguments
	from right to left

	USAGE

	const add3 = (n) => n + 3;
	const mul2 = (n) => n * 2;
	const add3mul2 = compose(add3, mul2);
	add3mul2(1) === 5; // add3(mul2(1)) === add3(2) === 5

	USAGE WITH MAP

	compose(add, map, Maybe)(5);

*/

functional.compose = function(...functions) {
	return function(...args) {
		return functions.reduceRight(function(result, fn, i) {
			return i === functions.length - 1 ? result : fn(result);
		}, functions[functions.length - 1].apply(null, args));
	};
};