const sinon = require('sinon');
const { Resolve, Reject } = require('../Either');
const { expect } = require('chai');

describe('Either', function() {

	const bigNumber = (n) => n > 99 ? Resolve(n) : Reject('Number is not big');

	describe('Reject', function() {

		it('map is not called', function() {
			const spy = sinon.spy();
			bigNumber(2).map(spy);
			expect(spy.callCount).to.equal(0);
		});

		it('throws when Reject contains non-string', function() {
			const bad = (n) => n > 99 ? Resolve(n) : Reject(new Error('this should throw'));
			expect(() => bad(2)).to.throw();
		});

		it('map can be chained', function() {
			const spy = sinon.spy();
			bigNumber(2).map(spy).map(spy).map(spy);
			expect(spy.callCount).to.equal(0);
		});

		it('orElse is called', function() {
			const spy1 = sinon.spy();
			const spy2 = sinon.spy();
			bigNumber(2).map(spy1).orElse(spy2);
			expect(spy1.callCount).to.equal(0);
			expect(spy2.callCount).to.equal(1);
		});

		it('can be called to lift value', function() {
			expect(bigNumber(2)()).to.equal('Number is not big');
		});

	});

	describe('Resolve', function() {

		it('map is called', function() {
			const spy = sinon.spy();
			bigNumber(100).map(spy);
			expect(spy.callCount).to.equal(1);
		});

		it('map is called with value', function() {
			bigNumber(100).map((x) => expect(x).to.equal(100));
		});

		it('orElse is not called', function() {
			const spy = sinon.spy();
			bigNumber(100).orElse(spy);
			expect(spy.callCount).to.equal(0);
		});

		it('can be called to lift value', function() {
			expect(bigNumber(100)()).to.equal(100);
		});

	});

});