/* eslint-disable no-sparse-arrays */
const List = require('../List');
const { expect } = require('chai');

describe('List', function() {

	describe('append', function() {

		it('appends an item to an array', function() {
			const x = [1, 2, 3];
			expect(List.append(4)(x)).to.deep.equal([1, 2, 3, 4]);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.append(4)(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('compact', function() {

		it('removes nulls and undefineds from array', function() {
			const x = [null, null, 1,,, 2, null, 3, undefined];
			expect(List.compact(x)).to.deep.equal([1, 2, 3]);
		});

		it('is immutable', function() {
			const x = [null, null, 1,,, 2, null, 3, undefined];
			List.compact(x);
			expect(x).to.deep.equal([null, null, 1,,, 2, null, 3, undefined]);
		});

	});

	describe('filter', function() {

		it('filters an array', function() {
			const x = [1, 2, 3, 4, 5, 6];
			const evens = (n) => n % 2 === 0;
			expect(List.filter(evens)(x)).to.deep.equal([2, 4, 6]);
		});

		it('is immutable', function() {
			const x = [1, 2, 3, 4, 5, 6];
			const evens = (n) => n % 2 === 0;
			List.filter(evens)(x);
			expect(x).to.deep.equal([1, 2, 3, 4, 5, 6]);
		});

	});

	describe('head', function() {

		it('returns first item', function() {
			const x = [1, 2, 3];
			expect(List.head(x)).to.equal(1);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.head(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('indexOf', function() {

		it('returns index of item in array', function() {
			const x = [1, 2, 3];
			expect(List.indexOf(2)(x)).to.equal(1);
			expect(List.indexOf(9)(x)).to.equal(-1);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.indexOf(2)(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('includes', function() {

		it('returns true or false', function() {
			const x = [1, 2, 3];
			expect(List.includes(2)(x)).to.equal(true);
			expect(List.includes(9)(x)).to.equal(false);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.includes(2)(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('insertAt', function() {

		it('returns spliced array', function() {
			const x = [1, 2, 3];
			expect(List.insertAt(1)('foo')(x)).to.deep.equal([1, 'foo', 2, 3]);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.insertAt(1)('foo')(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('isEmpty', function() {

		it('returns true or false', function() {
			const x = [1, 2, 3];
			const y = [, , , ];
			const z = [];
			expect(List.isEmpty(x)).to.equal(false);
			expect(List.isEmpty(y)).to.equal(false);
			expect(List.isEmpty(z)).to.equal(true);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.isEmpty(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('join', function() {

		it('returns string', function() {
			const x = [1, 2, 3];
			const y = [];
			expect(List.join('...')(x)).to.equal('1...2...3');
			expect(List.join('...')(y)).to.equal('');
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.join('...')(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('last', function() {

		it('returns last item', function() {
			const x = [1, 2, 3];
			expect(List.last(x)).to.equal(3);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.last(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('length', function() {

		it('returns length of array', function() {
			const x = [1, 2, 3];
			expect(List.length(x)).to.equal(3);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.length(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('prepend', function() {

		it('prepends an item to an array', function() {
			const x = [1, 2, 3];
			expect(List.prepend(0)(x)).to.deep.equal([0, 1, 2, 3]);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.prepend(0)(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('reduce', function() {

		it('reduces an array', function() {
			const x = [1, 2, 3, 4, 5];
			const sum = (y, z) => y + z;
			expect(List.reduce(sum)(x)).to.equal(15);
		});

		it('is immutable', function() {
			const x = [1, 2, 3, 4, 5];
			const sum = (y, z) => y + z;
			List.reduce(sum)(x);
			expect(x).to.deep.equal([1, 2, 3, 4, 5]);
		});

	});

	describe('reduceWithInitial', function() {

		it('reduces an array', function() {
			const x = [1, 2, 3, 4, 5];
			const sum = (y, z) => y + z;
			expect(List.reduceWithInitial(sum)(10)(x)).to.equal(25);
		});

		it('is immutable', function() {
			const x = [1, 2, 3, 4, 5];
			const sum = (y, z) => y + z;
			List.reduceWithInitial(sum)(10)(x);
			expect(x).to.deep.equal([1, 2, 3, 4, 5]);
		});

	});

	describe('reverse', function() {

		it('reverses an array', function() {
			const x = [1, 2, 3];
			expect(List.reverse(x)).to.deep.equal([3, 2, 1]);
		});

		it('is immutable', function() {
			const x = [1, 2, 3];
			List.reverse(x);
			expect(x).to.deep.equal([1, 2, 3]);
		});

	});

	describe('slice', function() {

		it('returns slice of array', function() {
			const x = [1, 2, 3, 4, 5];
			expect(List.slice(2)(4)(x)).to.deep.equal([3, 4]);
		});

		it('is immutable', function() {
			const x = [1, 2, 3, 4, 5];
			List.slice(2)(4)(x);
			expect(x).to.deep.equal([1, 2, 3, 4, 5]);
		});

	});

	describe('sort', function() {

		it('sorts array', function() {
			const x = [2, 5, 3, 1, 4];
			const byValue = (y, z) => y - z;
			expect(List.sort(byValue)(x)).to.deep.equal([1, 2, 3, 4, 5]);
		});

		it('is immutable', function() {
			const x = [2, 5, 3, 1, 4];
			const byValue = (y, z) => y - z;
			List.sort(byValue)(x);
			expect(x).to.deep.equal([2, 5, 3, 1, 4]);
		});

	});

});