const Maybe = require('../Maybe');
const { expect } = require('chai');

describe('Maybe', function() {

	it('is safe to map over Nothing', function() {
		const x = Maybe(null);
		x.map(function() {
			throw new Error('should not throw');
		});
	});

	it('is a container', function() {

		const a = Maybe();
		expect(a()).to.equal(undefined);

		const b = Maybe(null);
		expect(b()).to.equal(null);

		const c = Maybe(123);
		expect(c()).to.equal(123);

	});

	it('can be modified', function(done) {

		const x = Maybe(null);
		expect(x()).to.equal(null);

		x.map(function() {
			throw new Error('should not throw');
		});

		x('foo');

		expect(x()).to.equal('foo');

		x.map(function(value) {
			expect(value).to.equal('foo');
			done();
		});

	});

	it('can be chained', function(done) {

		const square = (x) => x * x;
		const add5 = (x) => x + 5;

		const x = Maybe(5);
		x.map(square).map(add5).map(function(value) {
			expect(value).to.equal(30);
			done();
		});

	});

	it('can be chained as Nothing', function() {

		const square = (x) => x * x;
		const add5 = (x) => x + 5;

		const x = Maybe(null);
		x.map(square).map(add5).map(function() {
			throw new Error('should not throw');
		});

	});

});