const Maybe = require('../Maybe');
const sinon = require('sinon');
const { expect } = require('chai');
const { stdout } = require('test-console');
const { log, map, compose } = require('../');

describe('core', function() {

	describe('log', function() {
		it('is chainable', function() {
			const x = (y) => new Promise((fn) => fn(y));
			const add5 = (n) => n + 5;
			const inspector = stdout.inspect();
			return x(5).then(log).then(add5).then(log).then(function() {
				inspector.restore();
				expect(inspector.output).to.deep.equal(['5\n', '10\n']);
			}).catch(function(err) {
				throw err;
			});
		});
	});

	describe('map', function() {

		it('works on Maybe', function() {
			const spy = sinon.spy();
			const x = Maybe(5);
			const y = Maybe(null);
			const assert5 = (z) => expect(z).to.equal(5);
			map(assert5)(x);
			map(spy)(y);
			expect(spy.callCount).to.equal(0);
		});

		it('works on Array', function() {
			const x = [1, 2, 3];
			const square = (n) => n * n;
			expect(map(square)(x)).to.deep.equal([1, 4, 9]);
		});

	});

	describe('compose', function() {

		it('composes one function', function() {
			const add3 = (y) => y + 3;
			expect(compose(add3)(10)).to.equal(13);
		});

		it('composes two functions', function() {
			const upper = (str) => str.toUpperCase();
			const exclaim = (str) => str.concat('!');
			expect(compose(exclaim, upper)('foo')).to.equal('FOO!');
		});

		it('composes three functions', function() {
			const double = (x) => x * 2;
			const add3 = (x) => x + 3;
			const add1 = (x) => x + 1;
			expect(compose(double, add3, add1)(2)).to.equal(12);
		});

		it('accepts more than one argument to innermost function', function() {
			const double = (x) => x * 2;
			const add = (x, y) => x + y;
			expect(compose(double, add)(4, 5)).to.equal(18);
		});

	});

});